/*
    RepCntl.CC
    Author...: Ken Mayer
    Date.....: January 24, 1999

    Similar to FormCntrl.CC in the dUFLP Library, 
    this contains default custom controls used by the
    author for his reports. The advantage, as with all custom
    controls, is that you can make a single change to all 
    your reports in one place ...

    Controls included at this time:

    kmTitleText1 -- A larger font text object used for a main title
                    on a report
    kmTitleText2 -- A slightly smaller font text object used for
                    a secondary title on a report.
    kmStandardText -- base class for most of the following
                    text controls
    kmDateText   -- prints "Date:" followed by the current date
    kmFieldHeadingText -- Bold text used to display a field
                        heading
    kmFieldHeadingBorderText
    kmFieldText  -- Used to display actual field, need to set
                    the codeblock appropriately ...
    kmNumFieldText 
                 -- Subclassed from kmFieldText, this
                    displays negative numbers as red ...
    kmCheckBoxText
    kmShape -- base class for other shape controls
       kmRoundedRectangleShape
       kmRectangleShape 
       kmEllipseShape
       kmCircleShape
       kmRoundedSquareShape
       kmSquareShape
    kmLine       -- basic line object

    **************************************************************
    Note I have no containers. From Charles Overbeck (on the
    development team for VdBASE 7.x):

    "The container cannot have a band as its parent.  The problem 
    is that the container is currently a windowed control, and 
    we don't allow windowed controls in bands (because the band 
    is potentially rendered multiple times, but there is only 
    one window--so how do you display one window several times?)."
    **************************************************************

*/

/*
    These definitions are used for the text controls below. 
    If you want to change a fontName, fontSize or a colorNormal
    property, do it here:
*/
#define STANDARDFONT     "Arial"
#define STANDARDSIZE     10
#define STANDARDCOLOR    "BLACK"
#define STANDARDNEGCOLOR "RED"
#define CHECKBOXFONT     "Wingdings"
#define CHECKBOXSIZE     12
#define TITLE1FONT       "Arial"
#define TITLE1SIZE       18
#define TITLE1COLOR      "BLACK"
#define TITLE2FONT       "Arial"
#define TITLE2SIZE       14
#define TITLE2COLOR      "BLACK"
#define SHAPECOLORS      "BLACK/WHITE"

/*
   Standard text placed on the pageTemplate, appears as a
   header at the top of each page. (If unclear on any
   of the properties, see onLine help)
*/
class kmTitleText1(parentObj) of TEXT(parentObj) custom
   with (this)
      alignHorizontal        := 1	// Center
      alignVertical          := 1	// Middle
      border                 := false
      borderStyle            := 0 // default
      colorNormal            := TITLE1COLOR
      fixed                  := false
      fontBold               := false
      fontItalic             := true
      fontName               := TITLE1FONT
      fontSize               := TITLE1SIZE
      fontStrikeout          := false
      fontUnderline          := false
      height                 := 0.3229
      leading                := 0.00
      marginHorizontal       := 0.00
      marginVertical         := 0.00
      metric                 := 3	// Inches
      prefixEnable           := false
      rotate                 := 0 // normal
      text                   := "This is a Main Report Title"
      tracking               := 0.00
      trackJustifyThreshhold := 0.00
      transparent            := true
      variableHeight         := false
      verticalJustifyLimit   := 0.00
      width                  := 3.5313
      wrap                   := true
   endwith

endclass

/*
   Note that this is subclassed from above, meant
   to be a smaller item under the first ...
*/
class kmTitleText2(parentObj) of kmTitleText1(parentObj) custom
   with (this)
      colorNormal     := TITLE2COLOR
      fontItalic      := false
      fontName        := TITLE2FONT
      fontSize        := TITLE2SIZE
      height          := 0.2396
      text            := "This is a secondary title"
   endwith

endclass

/*
   this is a standard text control that most of the others
   here are subclassed from ...
*/
class kmStandardText(parentObj) of TEXT(parentObj) custom
   with (this)
      alignHorizontal        := 0 // Left
      alignVertical          := 0 // Top
      border                 := false
      borderStyle            := 0 // default
      colorNormal            := STANDARDCOLOR
      fixed                  := false
      fontBold               := false
      fontItalic             := false
      fontName               := STANDARDFONT
      fontSize               := STANDARDSIZE
      fontStrikeout          := false
      fontUnderline          := false
      height                 := 0.18
      leading                := 0.00
      marginHorizontal       := 0.00
      marginVertical         := 0.00
      metric                 := 3	// Inches
      prefixEnable           := false
      rotate                 := 0 // normal
      text                   := "kmStandardText"
      tracking               := 0.00
      trackJustifyThreshhold := 0.00
      transparent            := true
      variableHeight         := false
      verticalJustifyLimit   := 0.00
      width                  := 1.1667
   endwith
endclass


/*
   Basic text and date combination ... nice thing about
   VdBASE 7 is the auto casting of types, so that
   we can display the date without having to add the DTOC()
   function ... of course, if you want more detail, like
   character month, and such, you would need to code for
   it ...
*/
class kmDateText(parentObj) of kmStandardText(parentObj) custom
   with (this)
      alignHorizontal        := 1 // center
      alignVertical          := 1	// Middle
      text                   := {||"Date: "+date()}
   endwith
endclass

/* 
   want the text on this one to be bold and line up on the
   right side ...
*/
class kmFieldHeadingText(parentObj) of kmStandardText(parentObj) custom
   with (this)
      height          := 0.18
      width           := 0.75
      alignHorizontal := 2 // Right
      text            := "FieldHeading: "
      fontBold        := true
   endwith
endclass

/*
   field heading in a border ... center it in the border.
*/
class kmFieldHeadingBorderText(parentObj) of kmFieldHeadingText(parentObj) custom
   with (this)
      alignHorizontal := 1 // Center
      alignVertical   := 1 // Middle
      border          := true
      borderStyle     := 4
      text            := "Heading in Border"
      width           := 1.5
   endwith
endclass

/*
    FieldText: Suggestion -- place this on the report,
    and then bring up the text property in the codeblock
    editor. Standard text controls are "datalinked" by
    the TEXT codeblock:

      form.streamSource1.rowset.fields["fieldName"].value

    So, replace "field codeblock" with the above,
    and of course insert the proper fieldName, and
    you should be fine.

    In addition, you may want to pass the field through
    the string object, particularly if you have
    the potential of nulls and you are doing a
    calculated field, so your codeblock *might* look
    like:

     new String( form.streamSource1.rowset.fields["fieldName"].value ) +;
     " " + ;
     new String( form.streamSource1.rowset.fields["fieldName2"].value )

*/
class kmFieldText(parentObj) of kmStandardText(parentObj) custom
   with (this)
      width           := 1.2188
      variableHeight  := true
      // This needs to be modified for each object ... obviously
      text            := {||"field codeblock"}
   endwith

endclass

/*
     This one is able to change it's color based
     on the value of a numeric field -- if it's negative, 
     we want to show the value in red, otherwise in black ...

     // See commented code below for a bit about the onOpen
     // event, details explained more fully in the kmCheckBoxText
     // class ...
*/
class kmNumFieldText( parentObj ) of kmFieldText( parentObj ) custom
   with (this )
      picture         := "99,999.99" // set appropriate picture
   endwith

   // you may want to set your own onOpen event, that 
   // stores the codeblock you're using for this field,
   // such as (do this using the inspector, go to the
   // events page ...):
   /*
      function kmNumFieldText1_onOpen
         this.text = {|| form.streamSource1.rowset.fields["fieldName"].value }
   */

   function canRender
      // Save codeblock, so we can evaluate it each time:
      if type( "this.text" ) == "CB"
         this.textCB = this.text
      endif

      /* 
         Need to evaluate the codeblock, and if it's numeric
         we need to check for a negative value ...
      */
      if type( "this.textCB()" ) $ "N,F" // numeric or float
         this.colorNormal = iif( this.textCB() < 0, STANDARDNEGCOLOR,;
                                                    STANDARDCOLOR )
      endif

   return true
endclass


/*
   Being able to display a checkbox takes a bit of work
   for those logical fields, but it can be done.
   This text control is subclassed from the standard
   one, rather than the kmFieldText object above,
   because I'll be using a different font, and such.

   // -------------------------------------------------
   // VERY IMPORTANT: You NEED to set your own onOpen 
   // event that stores the codeblock you're using 
   // for this field, such as (do this using the 
   // inspector, go to the events page ... replacing 
   // "fieldName" with your field):
   // -------------------------------------------------
   /*
      function kmCheckBoxText1_onOpen
         this.text = {|| form.streamSource1.rowset.fields["fieldName"].value }

         // if using a datamodule:
         // {|| form.datamodRef1.ref.queryName1.rowset.fields["fieldName"].value }
         // replace "queryName1" with the actual query name,
         // as well as the fieldName.
   */
   
*/
class kmCheckBoxText(parentObj) of kmStandardText(parentObj) custom
   with (this)
      height    := 0.1708
      width     := 0.1563
      fontName  := CHECKBOXFONT
      fontSize  := CHECKBOXSIZE
      text      := chr(180) // question mark in a 
                                  // diamond shape
   endwith


   function canRender
      // Save codeblock, so we can evaluate it each time:
      if type( "this.text" ) == "CB"
         this.textCB = this.text
      endif

      /* 
         if we get a 'true' out of it, return a checkbox with
         a check in it, otherwise, return an empty checkbox.
         Note that this assumes Wingdings for the font. If you
         want to use a different font, you will need to change
         the characters used ... use the CHARMAP utility
         that ships with some versions of Windows, or other
         Utilities you may find out there ...
      */
      if type( "this.textCB()" ) == "L"
         this.text = iif( this.textCB(), chr(254), chr(168) )
      else
         this.text = chr(180) // question mark in a diamond shape
      endif

   return true

endclass

/*
   The standard Rectangle doesn't function properly -- this is a
   bug in the software -- if you render the report on screen,
   a rectangle works like you might expect. However, if you
   print it (at least to the several HP Laserjets I've tried
   it with), the rectangle will print offset (up/left) from
   where it should. The shape object, however, doesn't do this --
   so you can generate a rectangle using the shape and have it
   work properly.
*/

// Start with a base shape class:
class kmShape(parentObj) of SHAPE(parentObj) custom
   with (this)
      width       := 2.9479
      height      := 0.4063
      metric      := 3	// Inches
      colorNormal := SHAPECOLORS
      penStyle    := 0 // Solid
      penWidth    := 2
   endwith
endclass

class kmRoundedRectangleShape(parentObj) of kmShape(parentObj) custom
   with (this)
      shapeStyle  := 0	// Rounded Rectangle
   endwith
endclass

class kmRectangleShape(parentObj) of kmShape(parentObj) custom
   with (this)
      shapeStyle  := 1	// Rectangle
   endwith
endclass

class kmEllipseShape(parentObj) of kmShape(parentObj) custom
   with (this)
      shapeStyle  := 2	// Ellipse
   endwith
endclass

class kmCircleShape(parentObj) of kmShape(parentObj) custom
   with (this)
      shapeStyle  := 3	// Circle
   endwith
endclass

class kmRoundedSquareShape(parentObj) of kmShape(parentObj) custom
   with (this)
      shapeStyle  := 4	// Rounded Square
   endwith
endclass

class kmSquareShape(parentObj) of kmShape(parentObj) custom
   with (this)
      shapeStyle  := 5	// Square
   endwith
endclass

/*
    Basic line, but I set the width to 2, because
    when photocopying, I've found that a width
    of 1 often gets lost ...
*/
class kmLine(parentObj) of LINE(parentObj) custom
   with (this)
      left            := 0
      right           := 4.4722
      top             := 0
      bottom          := this.top
      fixed           := false
      width           := 1
      metric          := 3	// Inches
      colorNormal     := STANDARDCOLOR
   endwith

endclass

/*
    ------------------------------------
    End of file: RepCntl.CC
    ------------------------------------
*/
