Class TMPQuery of Query custom
*----------------------
*- Name.....Temporary Table Query
*- Author...Todd Kreuter
*- For......dB2K, 7.5, 7.01
*- Usage....myQuery = new TMPQuery()

*- Description
   // For creating and working with temporary tables

   // Table is created using the the internal fields array,
   // Fields are added to the fields array using the arrays
   // Add(cField, bSQL) mehtod (See internal mehtod notes).

   // Can Use Short Hand syntax or standard SQL syntax
   // for creating fields. If the Short Hand syntax is used,
   // fields are automatically delimited, so there is no need
   // to manually delimit any thing when creating field names
   // containing spaces or SQL reserved keywords, and when
   // using tableNames which need to be delimited.

	// If using an overridden canOpen event to create the
   //	the table, return a call to TMPQuery::canOpen()

   //	Example: Creating table using canOpen event code:
   //		with (this.fields)
   //			add("ID C8")
   //			add("Name C30")
   //			add("Sales N12,2")
   //			add("Current B")
   //		endwith
   //
	//		this.createTable()
   //		this.createIndex("ID", "ID")
   //
   //		return TMPQuery::canOpen()

   // Because canOpen and onClose events have a specific use, 
   // you should avoid opening and closing the query during
   // the life of the query.  Open it once, close it once.

   // Use the custom methods to perform tasks which require
   // that the query be closed (but don't cloes the query).
	// When custom methods are called, the canOpen and onClose
   // events are disabled, the query is closed, the method
   // is executed, the query is opened, and the canOpen and 
   // onClose events are enabled.

   // Do not assign a database to these queries

*- Aknowledgements
// Gary White	-	Database methods can be used on tables
	//					located outside of the database path, if we
   //					provide the tables fullPath to those methods.
   //					API Code for retreiving OS Temporary path (NG)

*- Custom Properties
// skeleton		-	Skeleton for table name, ie "_TMP????"
// userDefPath	- 	Allow user to specify table path.
	//					Uses getDirectory() prompt
   //					This overrides useTempPath setting
// useTempPath	-	Determine if table is created in the OS 
	//					temporary directory, default is true.
   //					Set to false to create in current directory.
// dropOnClose	-	Delete table when query closed
   //					Only Works when setting query.active = false
   //             This is disabled when closed internally

*- Custom Methods
// CreateTable			- Create table from fields array
// CreateFromQuery	- Create table from existing query using UpdateSet
// CreateIndex			- Creates Index Tag (tagName, tagExpression)
// EmptyTable			- Empty table
// PackTable			- Packs table
// DropTable			- Deletes table file

*- Internal Properties
// fullPath	-	String containing full path necessary to locate
	//			  	the table, being the path + the tableName.
// fields	- 	Array containing fields list for createTable

*- Internal Methods
// AddField		Overridden fields array Add() Method
   //		  		Call fields.add(cField, <bSQL>) method
   //         	cField can be Short Hand syntax, "Name C30"
   //         	cField can be SQL Syntax "Name Char(30)"

   //         	Short Hand is the default, so when using
   //         	SQL syntax, bSQL must be true to bypass
   //         	the short hand converstion
// SetFullPath	Sets the path and file name
// SetActive	Open / closes the query

*- Internal Events
// canOpen	-	Sets sql statement for fullPath
// onClose	-	Delete table if dropOnClose is true

*- History
// 2001.0121 -	Original
// 2001.0201 -	Added Short Hand data type conversion
// 2001.0203 -	Improved error handling, messages
// 2001.0501 -	Disable canOpen event when internally
   //			 	reopening query
// 2001.0517 -	Added useTempPath property
   //				Added fullPath property, getFullPath method
   //				Included API call to retreive temporary path
//	2001.1004 - Added CreateFromQuery Method (uses updateSet)
// 2002.0130 - Added Skeleton property for customizing table name
// 2002.0130 - Added userDefPath property for allowing user to define path
// 2002.0130 - Removed tableName property
// 2002.0130 - Added setActive mehtod for internally setting
	//				active property when needed.
*----------------

   *- Custom
	this.skeleton		= "_TMP????"
	this.userDefPath	= false
	this.useTempPath	= true
   //this.useTempPath	= false

   this.tableName = ""
   this.dropOnClose	= true

   *- Internal
   this.fullPath		= null
   this.fields       = new array()
   this.fields.add   = class::addField

   function canOpen
   ****************
   *- Type:		Internal
   *- Purpose:	Sets the sql for the query
   *= Note:		Call TMPQuery::CanOpen() when overridden

   	local bCan
      bCan = NOT EMPTY(this.fullPath)
      this.sql	= [Select * from '] + this.fullPath + [']

     	return bCan

   function onClose
   ****************
   *- Type:		Internal
   *- Purpose:	Drop table when query closed, when applicable
   *- Note:		Only fires when set query.active to false

      if this.dropOnClose
         this.dropTable()
      endif
      return

   function AddField(cField, bSQL)
   *****************
   *- Type:		Internal
   *- Purpose:	Adds element to fields array
   *- Params:	cField, field Definition
   	//			bSQL, cField is in SQL Syntax (no conversion)

      if bSQL
         return ARRAY::ADD(cField)
      endif

      local nAt, cName, cType, cMap, cSql

      nAt   = cField.LastIndexOf(" ")
      cName = cField.left(nAt).rightTrim()
      cType = cField.right(cField.length - nAt).leftTrim()
      cMap  = cType.left(1)
      cSql  = null

      // Get SQL Data Type
      do case
      case cMap = "C"   // Character(n)
         cSql = "Char("

      case cMap = "N"   // Numeric(x,y)
         cSql  = "Numeric("

      case cMap = "D"   // Date
         cSql = "Date"

      case cMap = "B"   // Boolean<logical>
         cSql = "Boolean"

      case cMap = "A"   // Autoincrement
         cSql = "AutoInc"

      case cMap = "T"   // Timestamp
         cSql = "TimeStamp"

      case cMap = "L"   // Long Integer
         cSql = "Integer"

      case cMap = "S"   // Small Integer
         cSql = "SmallInt"

      case cMap = "F"   // Float(x,y)
         cSql = "Float("

      case cMap = "O"   // BLOB(n,t)
         cSQL = "BLOB("

      otherWise
         local cMsg
         cMsg  = "Invalid Data Type Conversion!" + chr(13)
         cMsg += "Field Name: " + cName + chr(13)
         cMsg += "Short Hand: " + cType + chr(13)

         MSGBOX(cMsg, "Cannot Add Field", 48)

      endcase

      if cSQL # Null

         // Substitue SQL Type
         cType = cType.stuff(0, 1, cSql)

         if cType.indexOf("(") > 0
            cType += ")"
         endif

         // Field Definition string
         cField = [''.'] + cName + [' ] + cType

         return ARRAY::ADD(cField)

      endif
      return

   function CreateTable
   ********************
   *- Type:		Custom
   *- Purpose:	Creates Table from fields array

      try

			if this.setFullPath()

            local cTable, i, cField
            private cMacro

            cTable = this.fullPath
            cTable = cTable.right(cTable.length - cTable.lastIndexOf("\") - 1)
            i = 0

            if this.fields.size = 0
               THROW Exception
            endif

            if this.database.tableExists(this.fullPath)
               this.database.dropTable(this.fullPath)
            endif

            cMacro = [create table "] + this.fullPath + [" (]

            for i = 1 to this.fields.size

               if this.fields[i].left(2) = "''"
                  // Short Hand Syntax, insert tableName 
                  cField = this.fields[i].stuff(1, 0, cTable)

               else
                  // SQL Syntax
                  cField = this.fields[i]
               endif

               cMacro += cField

               if i # this.fields.size
                  cMacro += ", "
               endif

            endfor

            cMacro += ")"

            &cMacro.
         endif

      catch(exception e)

         local cMsg
         if i = 0
            cMsg = "Expecting at Least One Field Definition!"
         else
            cMsg  = e.message
         endif

         MSGBOX(cMsg, "Cannot Create Table", 16)

         this.fullPath = null

      endtry

      return

	function CreateFromQuery(qSource, bIndex)
   ************************
   *- Type:		Custom
   *- Purpose:	Creates Table from existing Query
   *- Params:	qSource - Source query
   	//			bIndex - Whether to create indexes used in Source

		*- Create the Table
		local bActive, uObj

		this.setFullPath()

      bActive = qSource.active

      if NOT bActive
      	qSource.active = true
      endif

      qSource.rowset.first()

      uObj = new updateset()
      uObj.source = qsource.rowset
      uObj.destination = this.fullpath
      uObj.copy()

      qSource.active = bActive

      *- Create the Indexes
      if bIndex
      	local t, dIndex

         t = new tabledef()
         t.tableName	= qSource.rowset.tableName
         t.database	= qSource.database
         t.load()

         if t.indexes.size > 0
            for i = 1 to t.indexes.size
               dIndex = new DBFIndex()
               dIndex.descending		= t.indexes[i].descending
               dIndex.indexName		= t.indexes[i].indexName
               dIndex.expression		= t.indexes[i].expression
               dIndex.forExpression = t.indexes[i].forExpression
               dIndex.unique			= t.indexes[i].unique

               _app.databases[1].createIndex(this.fullPath, dIndex)
            endfor
         endif

      endif

      return

   function CreateIndex(tagName, tagExpression)
   ********************
   *- Type:		Custom
   *- Purpose:	Create an index tag
   *- Params:	tagName - Index Name
   	//			taxExpression - Index Expression
   *- Return:	Success on creating index tag (Logical)

      try

         local bCreate, bActive, dIndex
         bCreate	= true
         bActive	= this.active

         if PCOUNT() = 1
            // Assume one field tag
            tagExpression = tagName
         endif

			if bActive
				this.setActive(false)
         endif

         dIndex = new DBFIndex()
         dIndex.indexName  = tagName
         dIndex.expression = tagExpression
         this.database.createIndex(this.fullPath, dIndex)

			if bActive
         	this.setActive(true)
         endif

      catch(exception e)
         bCreate = false

         local cMsg
         cMsg  = e.message + chr(13)
         cMsg += "Tag Name  : " + tagName + chr(13)
         cMsg += "Expression: " + tagExpression

         MSGBOX(cMsg, "Cannot Create Index", 16)

      endtry

      return bCreate

   function PackTable
   ******************
   *- Type:		Custom
   *- Purpose:	Pack the table
   *- Return:	Success on packing table (Logical)

      try
         local bPack, bActive
         bPack		= true
         bActive  = this.active

         if bActive
				this.setActive(false)
			endif

         this.database.packTable(this.fullPath)

         if bActive
				this.setActive(true)
			endif
         this.rowset.refreshControls()

      catch(exception e)
         bPack = false

         MSGBOX(e.message, "Cannot Pack Table", 48)

      endtry

      return bPack

   function EmptyTable
   *******************
   *- Type:		Custom
   *- Purpose:	Empty the table
   *- Return:	Success on emptying table (Logical)

      try
         local bEmpty, bActive
         bEmpty	= true
         bActive  = this.active

			if bActive
				this.setActive(false)
         endif

         this.database.emptyTable(this.fullPath)

			if bActive
				this.setActive(true)
         endif

         this.rowset.refreshControls()

      catch(exception e)
         bEmpty = false

         MSGBOX(e.message, "Cannot Empty Table", 48)

      endtry

      return bEmpty

   function DropTable
   ******************
   *- Type:		Custom / Internal
   *- Purpose:	Deletes the file
   *- Return:	Success on dropping table (Logical)

      try
      	local bDrop
         bDrop = true

         this.active = false
         this.database.dropTable(this.fullPath)

      catch(exception e)
			bDrop = false

         MSGBOX(e.message, "Cannot Delete Table", 16)

      endtry

      return bDrop

	function setFullPath
   ********************
   *- Type:		Internal
   *- Purpose:	Sets path and file name of table, custom 
   	//			properties determine path and file name
   *- Return:	Success on setting full path (Logical)


   	local bPath, cPath, nLen, n
      bPath = true

		this.fullPath = null

		do case
      case this.userDefPath
      	cPath = getDirectory()

         if EMPTY(cPath)
         	bPath = false
         endif

      case this.useTempPath

         if type("GetTempPath") # "FP"
            extern culong GetTempPath(culong, cstring) kernel32 ;
            from "GetTempPathA"
         endif

         cPath	= SPACE(256)
         nLen	= cPath.length
         n		= GetTempPath(nLen, cPath)

         if n > nLen
            nLen	= n
            n		= GetTempPath(nLen, cPath)
         endif

         if n > 0
            cPath	= cPath.left(nLen)
         else
            cPath	= SET("Directory")
         endif

		otherwise
         cPath = SET("Directory")

      endcase

      if bPath
      	if cPath.right(1) # "\"
         	cPath += "\"
         endif


         if empty(this.tablename)
			   this.fullPath	= FUNIQUE(cPath + this.skeleton)
         else
            this.fullPath  = cPath + this.tableName
         endif

         if EMPTY(this.fullPath)
         	MSGBOX("Could Not Create Unique Table", "Canceled", 48)
         	bPath = false
         endif

		endif

	  	return bPath

	function setActive(bActive)
   ******************
   *- Type:		Internal
   *- Purpose:	For setting active property internally
   	//			To prevent canOpen and onClose events from firing
   *- Param:	bActive - Setting of query.active

      if bActive
      	this.active = true
         this.canOpen = class::canOpen
         this.onClose = class::onClose

      else
      	this.canOpen = null
         this.onClose = null
         this.active = false
      endif

      return

endclass