/*
    MOVER.CC
    Author: Ken Mayer
    Date..: 04/10/1998

    Updated: May 31, 2000 -- Eric Bachleitner
             Eric updated this so that mover is now
             self-contained -- i.e., you can have multiple
             mover controls on the same form, and more ...

    A custom container with two listboxes and some buttons
    and code to handle moving items between listboxes.

    Usage:

      First, place the custom container onto a form:
           set procedure to mover.cc additive
           // in designer drop a copy of MOVERCONTAINER
           // onto a form, add whatever other controls
           // you wish, and ...

      When working with the form that has an instance
      of MOVERCONTAINER, you must assign the source 
      array to this form's source array:

           set procedure to mover.wfm additive
           fMover = new moverform()
           fMover.SourceArray = myarrayreference
           fMover.ReadModal()

      That should be all -- the form will make a copy
      of your original source for use in the form, so
      that we don't destroy your original array.

      OPTIONAL:
         These two text controls are over the two
         listboxes -- you may wish to change the 
         text:

         fMover.moverContainer1.SourceTitle.text = "Something"
         fMover.moverContainer1.DestTitle.text   = "Somethingelse"



      =========================
      TO GET DESTINATION ARRAY:
      =========================
      To get the data from the destination array, use
      code similar to that below:

          if fMover.destArray[1] # "<Empty>"
             myNewArray = new Array()
             for i = 1 to fMover.destArray.size
                 mynewarray.add( fMover.destArray[i] )
             next
          else
             ? "Nothing to process"
          endif

     ----------
     SIDEFFECTS
     ----------
     NOTE: Three arrays are attached to the form:
     SourceArray -- assigned by the developer
     CopySource  -- a copy of the source -- this is what's
                    manipulated by the mover control
     DestArray   -- this is what's in the destination 
                    listbox, and is in theory what you're
                    looking for ...

*/

class MOVER2CONTAINER( oFormObj ) of CONTAINER( oFormObj ) custom
   with ( this )
      transparent := true
      width       := 40
      height      := 9.5
      borderStyle := 10	// Etched Out
      onOpen      := CLASS::OpenIt
   endwith

   this.SOURCETITLE = new TEXT(this)
   with (this.SOURCETITLE)
      height = 0.8
      left = 2
      top = 0.44
      width = 12
      border = true
      colorNormal = "b"
      alignVertical = 1	// Middle
      alignHorizontal = 1	// Center
      fontBold = true
      text = "Source"
      borderStyle = 1	// Raised
   endwith


   this.DESTTITLE = new TEXT(this)
   with (this.DESTTITLE)
      height = 0.8
      left = 25.5556
      top = 0.44
      width = 12
      border = true
      colorNormal = "b"
      alignVertical = 1	// Middle
      alignHorizontal = 1	// Center
      fontBold = true
      text = "Destination"
      borderStyle = 1	// Raised
   endwith


   this.SOURCELISTBOX = new LISTBOX(this)
   with (this.SOURCELISTBOX)
      onLeftDblClick = {;This.parent.MoveCurrentRightButton_onClick()}
      height = 7.6
      left = 1.2222
      top = 1.44
      width = 13.4444
      id = 101
      colorHighLight = "white/blue"
      dataSource = 'ARRAY {"Source Array"}'
      multiple = true
   endwith


   this.DESTLISTBOX = new LISTBOX(this)
   with (this.DESTLISTBOX)
      onLeftDblClick = {;This.parent.MoveCurrentLeftButton_onClick()}
      height = 7.6
      left = 25
      top = 1.44
      width = 13.4444
      id = 101
      colorHighLight = "white/blue"
      dataSource = 'ARRAY {"Dest. Array"}'
      multiple = true
   endwith


   this.BTNCONTAINER = new CONTAINER(this)
   with (this.BTNCONTAINER)
      transparent = true
      left = 15.7778
      top = 1.76
      width = 8
      height = 6.48
      borderStyle = 3	// None
   endwith


   this.BTNCONTAINER.MOVECURRENTRIGHTBUTTON = new PUSHBUTTON(this.BTNCONTAINER)
   with (this.BTNCONTAINER.MOVECURRENTRIGHTBUTTON)
      onClick = {;this.parent.parent.MOVECURRENTRIGHTBUTTON_ONCLICK()}
      height = 1.08
      left = 1.1111
      top = 0.6
      width = 6.1111
      text = ""
      speedBar = true
      upBitmap = "RESOURCE #854"
      speedTip = "Move Current Item(s) to Destination"
      pageno = 0
      value = false
   endwith


   this.BTNCONTAINER.MOVEALLRIGHTBUTTON = new PUSHBUTTON(this.BTNCONTAINER)
   with (this.BTNCONTAINER.MOVEALLRIGHTBUTTON)
      onClick = {;this.parent.parent.MOVEALLRIGHTBUTTON_ONCLICK()}
      height = 1.08
      left = 1.1111
      top = 1.72
      width = 6.1111
      text = ""
      speedBar = true
      upBitmap = "RESOURCE #855"
      speedTip = "Move All Item to Destination"
      pageno = 0
      value = false
   endwith


   this.BTNCONTAINER.MOVECURRENTLEFTBUTTON = new PUSHBUTTON(this.BTNCONTAINER)
   with (this.BTNCONTAINER.MOVECURRENTLEFTBUTTON)
      onClick = {;this.parent.parent.MOVECURRENTLEFTBUTTON_ONCLICK()}
      height = 1.08
      left = 1.1111
      top = 3.4
      width = 6.1111
      text = ""
      speedBar = true
      upBitmap = "RESOURCE #853"
      speedTip = "Move Current Item(s) to Source"
      pageno = 0
      value = false
   endwith

   this.BTNCONTAINER.MOVEALLLEFTBUTTON = new PUSHBUTTON(this.BTNCONTAINER)
   with (this.BTNCONTAINER.MOVEALLLEFTBUTTON)
      onClick = {;this.parent.parent.MOVEALLLEFTBUTTON_ONCLICK()}
      height = 1.08
      left = 1.1111
      top = 4.48
      width = 6.1111
      text = ""
      speedBar = true
      upBitmap = "RESOURCE #852"
      speedTip = "Move All Item to Source"
      pageno = 0
      value = false
   endwith

   function MOVEALLLEFTBUTTON_onClick
      /* 
         ----------------------------------------
         Move all items in destination back
         to source:
         ----------------------------------------
      */

      // First, check to see if we're working with an
      // "empty" array ...:
      if aLen( This.destArray ) == 1 and ;
         This.destArray[1] == "<Empty>"
         return
      endif

      // Copy the elements in the destination array
      // back to the source array -- fun part is
      // that there may be some elements already
      // there ...:
      if This.copySource[1] == "<Empty>"
         This.copySource = new Array()
      endif
      for i = 1 to aLen( This.destArray )
          This.copySource.add( This.destArray[i] )
      next
      class::ResetDestination() // it's now empty
      class::ReassertArrays()   // show 'em ...

   return

   function MOVEALLRIGHTBUTTON_onClick
      /* 
         ----------------------------------------
         Move all items in source to destination:
         ----------------------------------------
      */

      // First, check to see if we're working with an
      // "empty" array ...:
      if aLen( This.copySource ) == 1 and ;
         This.copySource[1] == "<Empty>"
         return
      endif

      // Copy the elements in the source array
      // to the destination array -- fun part is
      // that there may be some elements already
      // there ...:
      if This.destArray[1] == "<Empty>"
         This.destArray = new Array()
      endif
      for i = 1 to aLen( This.copySource )
          This.destArray.add( This.copySource[i] )
      next
      class::EmptySource() // it's now empty
      class::ReassertArrays()   // show 'em ...

   return

   function MOVECURRENTLEFTBUTTON_onClick
        /* 
         ----------------------------------------
         Move current item(s) in destination back
         to source (this is called both from
         a left-double-click on the listbox,
         and from the pushbutton named above):
         ----------------------------------------
      */

      // First, check to see if we're working with the
      // "empty" array ...:
      if aLen( This.destArray ) == 1 and ;
         This.destArray[1] == "<Empty>"
         return
      endif

      // Now for the tricky part -- moving just the
      // element(s) selected:
      if This.copySource[1] == "<Empty>"
         This.copySource = new array()
      endif

      cObjRef = "this.DestListBox"
      cSize = cObjRef+".selected().size"
      cArray = cObjRef+".selected()[i]"
      for i = 1 to &cSize.
          This.copySource.add( &cArray. )

          // delete this element from the listbox's array:
          nRow = This.destArray.scan( &cArray. )
          This.destArray.delete( nRow )
          This.destArray.size--
      next
      // if destination is now empty:
      if This.destArray.size = 0
         class::ResetDestination()
      endif
      class::ReassertArrays()

   return

   function MOVECURRENTRIGHTBUTTON_onClick
        /* 
         ----------------------------------------
         Move current item(s) in source
         to destination (this is called both from
         a left-double-click on the listbox,
         and from the pushbutton named above):
         ----------------------------------------
      */

      // First, check to see if we're working with the
      // "empty" array ...:
      if aLen( this.CopySource ) == 1 and ;
         this.CopySource[1] == "<Empty>"
         return
      endif

      // Now for the tricky part -- moving just the
      // element(s) selected:
      if this.destArray[1] == "<Empty>"
         this.destArray = new array()
      endif    

		cObjRef	= "This.SourceListBox"
      cSize  = cObjRef +".selected().size"
      cArray = cObjRef +".selected()[i]"
      for i = 1 to &cSize.
          this.destArray.add( &cArray. )

          // delete this element from the listbox's array:
          nRow = This.copysource.scan( &cArray. )
          this.copysource.delete( nRow )
          this.copysource.size--
      next
      // if source is now empty:
      if this.CopySource.size = 0
         class::EmptySource()
      endif

      class::ReassertArrays()
   return

   PROCEDURE OpenIt

      // open the container:
      if type( "this.SourceArray" ) = "U" or ;
         type( "this.SourceArray" ) # "A"
         msgbox( "Please assign an array to the "+;
                 "form's SourceArray property!" )
         form.close()
         return
      endif

      // Copy the source array:
      this.CopySource = new array()
      for i = 1 to alen( this.SourceArray )
          this.CopySource.add( this.SourceArray[ i ] )
      next

      // assign to source listbox:
      this.SourceListBox.datasource =;
                 "ARRAY this.CopySource"

      // assign a simple array to start
      // with to the destination listbox:
      this.DestArray = new Array()
      this.DestArray.add( "<Empty>" )
      this.DestListBox.datasource = "ARRAY this.DestArray"

   procedure resetDestination
      // reset destination array to empty:
      This.DestArray = new Array()
      This.DestArray.add( "<Empty>" )

      This.DestListBox.datasource = "ARRAY This.DestArray"

   procedure emptySource
      // if the source is empty:
      This.copySource = new Array()
      This.copySource.add( "<Empty>" )
      This.SourceListBox.datasource = "ARRAY This.copySource"

   procedure reassertArrays
      // Ensure that the listboxes are showing the correct
      // values in the arrays

      this.DestListBox.DataSource = "ARRAY This.DestArray"
		This.SourceListBox.DataSource = "ARRAY This.copySource"

endclass // MOVERCONTAINER

// -----------------------
// End of File: MOVER.CC
// -----------------------
